==> vradi-swing-0.0.1-bin.zip: org/sharengo/wikitty/FieldType.java <==
/* *##%
 * Copyright (c) 2009 Sharengo, Guillaume Dufrene, Benjamin POUSSIN.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *##%*/

package org.sharengo.wikitty;


import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 *
 * @author poussin
 * @version $Revision$
 *
 * Last update: $Date$
 * by : $Author$
 */
public class FieldType implements Serializable {

    static public enum TYPE {
        BOOLEAN, DATE, NUMERIC, STRING, WIKITTY;

        /**
         * convert string to TYPE, this methode accept not trimed and not well
         * cased string (difference with valueOf)
         * @param name
         * @return TYPE else exception is throw
         */
        public static TYPE parse(String name) {
            TYPE result = valueOf(name.trim().toUpperCase());
            return result;
        }
    }

    /** to use log facility, just put in your code: log.info(\"...\"); */
    static private Log log = LogFactory.getLog(FieldType.class);

	private static final long serialVersionUID = 1L;

	public static final int NOLIMIT = Integer.MAX_VALUE;

	TYPE type;
	int lowerBound;
	int upperBound;
    boolean unique = false;
    boolean notNull = false;

    /** used to store tag/value used by client side ex: editor=xhtml */
    Map<String, String> tagValues = new HashMap<String, String>();

    public FieldType() {
    }

    public FieldType(TYPE type, int lowerBound, int upperBound) {
        this.type = type;
        this.lowerBound = lowerBound;
        this.upperBound = upperBound;
    }

    public FieldType(TYPE type, int lowerBound, int upperBound, boolean unique, boolean notNull) {
        this.type = type;
        this.lowerBound = lowerBound;
        this.upperBound = upperBound;
        this.unique = unique;
        this.notNull = notNull;
    }

    public void addTagValue(String tag, String value) {
        tagValues.put(tag, value);
    }

    public String getTagValue(String tag) {
        String result = tagValues.get(tag);
        return result;
    }

    /**
     * Return true if this field have upperBound > 1
     * @return
     */
    public boolean isCollection() {
        return upperBound > 1;
    }

    /**
     * Return string definition for this field
     * @param name field name used for definition
     * @return
     */
    public String toDefinition(String name) {
        String result = type + " " + name;
        if (lowerBound != 0 || upperBound != 0) {
            if (upperBound != NOLIMIT) {
                result += "[" + lowerBound + "-" + upperBound + "]";
            } else {
                result += "[" + lowerBound + "-*]";
            }
        }
        if (isUnique()) {
            result += " unique";
        }
        if (isNotNull()) {
            result += " not null";
        }
        for (String tag : tagValues.keySet()) {
            result += " " + tag + "=" + tagValues.get(tag);
        }
        return result;
    }

    /**
     * Convert value in argument in right type for this FieldType. Don't support
     * collection
     * 
     * @param value value to convert
     * @return object in type of this FieldType
     */
    protected Object getContainedValidObject( Object value ) {
    	Object result = null;
        switch (type) {
        case DATE: 
        	result = WikittyUtil.toDate(value); break;
        case NUMERIC: 
        	result = WikittyUtil.toBigDecimal(value); break;
        case BOOLEAN:
        	result = WikittyUtil.toBoolean(value); break;
        case STRING:
        	result = WikittyUtil.toString(value); break;
        default:
            // if type is not found then type is business type
            // and is wikity object
            result = WikittyUtil.toWikitty(value); break;
        }
        return result;
    }
    
    /**
     * Return a valid value for this field.
     * @param value is casted if possible to an actual correct value.
     * @return
     */
    public Object getValidValue( Object value ) throws WikittyException {
        if (value == null && isNotNull()) {
            throw new WikittyException("Value can't be null for this field");
        }

        Object result;
        if (value == null) {
            result = null;
        } else if (isCollection()) {
    		if ( !(value instanceof Collection) ) {
    			throw new WikittyException( "A collection is expected for type " 
    					+ type.name() + "[" + lowerBound + " - " + upperBound + "]" );
    		}
    		Collection<Object> col;
            if (isUnique()) {
                col = new HashSet<Object>();
            } else {
                col = new ArrayList<Object>();
            }
 
            // copy all value in new collections
    		for ( Object o : (Collection) value ) {
    			col.add( getContainedValidObject(o) );
    		}
            result = col;
    	} else {
            result = getContainedValidObject(value);
    	}
        return result;
    }

    /**
     * test if value in argument is valide for this field type
     * @param value to test
     * @return true if value is valid
     */
    public boolean isValidValue(Object value) {
    	return getValidValue(value) != null;
    }

	public TYPE getType() {
		return type;
	}

    public int getLowerBound() {
        return lowerBound;
    }

    public int getUpperBound() {
        return upperBound;
    }

    public boolean isUnique() {
        return unique;
    }

    public boolean isNotNull() {
        return notNull;
    }
    
}

==> vradi-swing-0.0.3-bin.zip: org/sharengo/wikitty/FieldType.java <==
/* *##%
 * Copyright (c) 2009 Sharengo, Guillaume Dufrene, Benjamin POUSSIN.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *##%*/

package org.sharengo.wikitty;


import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 *
 * @author poussin
 * @version $Revision$
 *
 * Last update: $Date$
 * by : $Author$
 */
public class FieldType implements Serializable {

    static public enum TYPE {
        BOOLEAN, DATE, NUMERIC, STRING, WIKITTY;

        /**
         * convert string to TYPE, this methode accept not trimed and not well
         * cased string (difference with valueOf)
         * @param name
         * @return TYPE else exception is throw
         */
        public static TYPE parse(String name) {
            TYPE result = valueOf(name.trim().toUpperCase());
            return result;
        }
    }

    /** to use log facility, just put in your code: log.info(\"...\"); */
    static private Log log = LogFactory.getLog(FieldType.class);

	private static final long serialVersionUID = 1L;

	public static final int NOLIMIT = Integer.MAX_VALUE;

    public static String WORD_SEPARATOR = "_";

	TYPE type;
	int lowerBound;
	int upperBound;
    boolean unique = false;
    boolean notNull = false;

    /** used to store tag/value used by client side ex: editor=xhtml */
    Map<String, String> tagValues = new HashMap<String, String>();

    public FieldType() {
    }

    public FieldType(TYPE type, int lowerBound, int upperBound) {
        this.type = type;
        this.lowerBound = lowerBound;
        this.upperBound = upperBound;
    }

    public FieldType(TYPE type, int lowerBound, int upperBound, boolean unique, boolean notNull) {
        this.type = type;
        this.lowerBound = lowerBound;
        this.upperBound = upperBound;
        this.unique = unique;
        this.notNull = notNull;
    }

    public void addTagValue(String tag, String value) {
        tagValues.put(tag, value);
    }

    public String getTagValue(String tag) {
        String result = tagValues.get(tag);
        return result;
    }

    /**
     * Return true if this field have upperBound > 1
     * @return
     */
    public boolean isCollection() {
        return upperBound > 1;
    }

    /**
     * Return string definition for this field
     * @param name field name used for definition
     * @return
     */
    public String toDefinition(String name) {
        String result = type + " " + name.replace(" ", WORD_SEPARATOR);
        if (lowerBound != 0 || upperBound != 0) {
            if (upperBound != NOLIMIT) {
                result += "[" + lowerBound + "-" + upperBound + "]";
            } else {
                result += "[" + lowerBound + "-*]";
            }
        }
        if (isUnique()) {
            result += " unique";
        }
        if (isNotNull()) {
            result += " not null";
        }
        for (String tag : tagValues.keySet()) {
            result += " " + tag + "=" + tagValues.get(tag).replace(" ", WORD_SEPARATOR);
        }
        return result;
    }

    /**
     * Convert value in argument in right type for this FieldType. Don't support
     * collection
     * 
     * @param value value to convert
     * @return object in type of this FieldType
     */
    protected Object getContainedValidObject( Object value ) {
    	Object result = null;
        switch (type) {
        case DATE: 
        	result = WikittyUtil.toDate(value); break;
        case NUMERIC: 
        	result = WikittyUtil.toBigDecimal(value); break;
        case BOOLEAN:
        	result = WikittyUtil.toBoolean(value); break;
        case STRING:
        	result = WikittyUtil.toString(value); break;
        default:
            // if type is not found then type is business type
            // and is wikity object
            result = WikittyUtil.toWikitty(value); break;
        }
        return result;
    }
    
    /**
     * Return a valid value for this field.
     * @param value is casted if possible to an actual correct value.
     * @return
     */
    public Object getValidValue( Object value ) throws WikittyException {
        if (value == null && isNotNull()) {
            throw new WikittyException("Value can't be null for this field");
        }

        Object result;
        if (value == null) {
            result = null;
        } else if (isCollection()) {
    		if ( !(value instanceof Collection) ) {
    			throw new WikittyException( "A collection is expected for type " 
    					+ type.name() + "[" + lowerBound + " - " + upperBound + "]" );
    		}
    		Collection<Object> col;
            if (isUnique()) {
                col = new HashSet<Object>();
            } else {
                col = new ArrayList<Object>();
            }
 
            // copy all value in new collections
    		for ( Object o : (Collection) value ) {
    			col.add( getContainedValidObject(o) );
    		}
            result = col;
    	} else {
            result = getContainedValidObject(value);
    	}
        return result;
    }

    /**
     * test if value in argument is valide for this field type
     * @param value to test
     * @return true if value is valid
     */
    public boolean isValidValue(Object value) {
    	return getValidValue(value) != null;
    }

	public TYPE getType() {
		return type;
	}

    public int getLowerBound() {
        return lowerBound;
    }

    public int getUpperBound() {
        return upperBound;
    }

    public boolean isUnique() {
        return unique;
    }

    public boolean isNotNull() {
        return notNull;
    }
    
}
